<?php
/**
 * @file
 * Implements postmark support on behalf of Drupal core.
 */
define('POSTMARKAPP_API_KEY', variable_get('postmark_api_key', ''));

module_load_include('php', 'postmark', 'includes/Postmark');

/**
* Modify the drupal mail system to use smtp when sending emails.
* Include the option to choose between plain text or HTML
*/
class PostmarkMailSystem implements MailSystemInterface {

  protected $AllowHtml;
  /**
   * Concatenate and wrap the e-mail body for either
   * plain-text or HTML emails.
   *
   * @param $message
   *   A message array, as described in hook_mail_alter().
   *
   * @return
   *   The formatted $message.
   */
  public function format(array $message) {
    $this->AllowHtml = TRUE;
    $message['body'] = implode("\n\n", $message['body']);
    return $message;
  }

  /**
   * Send the e-mail message.
   *
   * @see drupal_mail()
   *
   * @param $message
   *   A message array, as described in hook_mail_alter().
   * @return
   *   TRUE if the mail was successfully accepted, otherwise FALSE.
   */
  public function mail(array $message) {
		static $postmark_mail;

		if (!isset($postmark_mail)) {
			$postmark_mail = new Mail_Postmark();
		}

		// Parse 'From' e-mail address.
		$address = postmark_parse_address($message['from']);
		$sender = $address[0]['mail'];

		$postmark_mail->replyTo($address[0]['mail']);
		$postmark_mail->from(variable_get('site_mail', ini_get('sendmail_from')));
		if ($address[0]['name'] != '') {
			$postmark_mail->fromName($address[0]['name']);
		}
		unset($message['headers']['From']);

		// Set up our debugging mode vars
		$debug_mode = variable_get('postmark_debug_mode', 0);
		$debug_email = variable_get('postmark_debug_email', '');
		if (!$debug_mode) {
			// Set recipients.
			foreach (postmark_parse_address($message['to']) as $id => $address) {
				// For the first email set as initial 'To'
				if ($id == 0) {
					$postmark_mail->to($address['mail'], $address['name']);
				}
				// Otherwise we must add to the array
				else {
					$postmark_mail->addTo($address['mail'], $address['name']);
				}
			}
		}
		else {
			// Reroute to debug e-mail address.
			if ($debug_email != '') {
				drupal_set_message(t('Debugging email used @email', array('@email' => $debug_email)), 'warning');
				$postmark_mail->to($debug_email);
			}
		}

		$postmark_mail->subject($message['subject']);
		// Check the header content type to see if email is plain text
		// if not we send as HTML
		if (strpos($message['headers']['Content-Type'], 'text/plain') !== FALSE) {
			if ($sender != variable_get('site_mail', ini_get('sendmail_from'))) {
				$message['body'] .= "\r\n\r\n";
				$message['body'] .= t('Message sent by').":\r\n";
				$message['body'] .= $sender;
			}

			$postmark_mail->messagePlain($message['body']);
		}
		else {
			if ($sender != variable_get('site_mail', ini_get('sendmail_from'))) {
				$message['body'] .= "<p>".t('Message sent by').":<br />".$sender."</p>";
			}
			$postmark_mail->messageHtml($message['body']);
		}

		// If debug mode is on, output the message
		if ($debug_mode) {
			drupal_set_message('Message array: <pre>'. print_r($message, TRUE) .'</pre>', 'warning');    
		}

		// If the debug option of not sending a credit, i.e.
		// for testing, is switched on just return TRUE,
		// otherwise send the email via Postmark
		if (variable_get('postmark_debug_no_send', 0)) {
			drupal_set_message('Email successfully tested, no email has been sent (no credits used)', 'warning');
			return TRUE;
		}
		else {
			try {
				if (!($result = $postmark_mail->send())) {
					watchdog('postmark', "Mail sending error: $postmark_mail->ErrorInfo", NULL, WATCHDOG_ERROR);
				}
			}
			catch (Exception $e) {
				watchdog('postmark', 'Exception message: '. $e->getMessage(), NULL, WATCHDOG_ERROR);
				drupal_set_message('Mail sending error: '. $e->getMessage(), 'error');
				// If debugging is on let's put the whole exception into watchdog
				// to enable closer inspection
				if ($debug_mode) {
					watchdog('postmark', 'Exception caught: <pre>'. print_r($e, TRUE) .'</pre>', NULL, WATCHDOG_ERROR);
				}
			}
		}
  return $result;
  }
  /**
   * Splits the input into parts based on the given boundary.
   *
   * Swiped from Mail::MimeDecode, with modifications based on Drupal's coding
   * standards and this bug report: http://pear.php.net/bugs/bug.php?id=6495
   *
   * @param input
   *   A string containing the body text to parse.
   * @param boundary
   *   A string with the boundary string to parse on.
   * @return
   *   An array containing the resulting mime parts
   */
  protected function _boundary_split($input, $boundary) {
    $parts       = array();
    $bs_possible = drupal_substr($boundary, 2, -2);
    $bs_check    = '\"' . $bs_possible . '\"';

    if ($boundary == $bs_check) {
      $boundary = $bs_possible;
    }

    $tmp = explode('--' . $boundary, $input);

    for ($i = 1; $i < count($tmp); $i++) {
      if (trim($tmp[$i])) {
        $parts[] = $tmp[$i];
      }
    }

    return $parts;
  }  //  End of _smtp_boundary_split().



  /**
   * Strips the headers from the body part.
   *
   * @param input
   *   A string containing the body part to strip.
   * @return
   *   A string with the stripped body part.
   */
  protected function _remove_headers($input) {
    $part_array = explode("\n", $input);

    if (strpos($part_array[0], 'Content') !== FALSE) {
      if (strpos($part_array[1], 'Content') !== FALSE) {
        if (strpos($part_array[2], 'Content') !== FALSE) {
            array_shift($part_array);
            array_shift($part_array);
            array_shift($part_array);
        }
        else {
          array_shift($part_array);
          array_shift($part_array);
        }
      }
      else {
        array_shift($part_array);
      }
    }

    $output = implode("\n", $part_array);
    return $output;
  }  //  End of _smtp_remove_headers().



  /**
   * Returns a string that is contained within another string.
   *
   * Returns the string from within $source that is some where after $target
   * and is between $beginning_character and $ending_character.
   *
   * @param $source
   *   A string containing the text to look through.
   * @param $target
   *   A string containing the text in $source to start looking from.
   * @param $beginning_character
   *   A string containing the character just before the sought after text.
   * @param $ending_character
   *   A string containing the character just after the sought after text.
   * @return
   *   A string with the text found between the $beginning_character and the
   *   $ending_character.
   */
  protected function _get_substring($source, $target, $beginning_character, $ending_character) {
    $search_start     = strpos($source, $target) + 1;
    $first_character  = strpos($source, $beginning_character, $search_start) + 1;
    $second_character = strpos($source, $ending_character, $first_character) + 1;
    $substring        = drupal_substr($source, $first_character, $second_character - $first_character);
    $string_length    = drupal_strlen($substring) - 1;

    if ($substring[$string_length] == $ending_character) {
      $substring = drupal_substr($substring, 0, $string_length);
    }

    return $substring;
  }  //  End of _smtp_get_substring().
}
